/*
 *  version.h
 */

#define LD10K1_LIB_MAJOR		0 /**< major number of library version */
#define LD10K1_LIB_MINOR		1 /**< minor number of library version */
#define LD10K1_LIB_SUBMINOR	8 /**< subminor number of library version */
/** library version */
#define LD10K1_LIB_VERSION		((LD10K1_LIB_MAJOR<<16)|\
					 (LD10K1_LIB_MINOR<<8)|\
				  	 LD10K1_LIB_SUBMINOR)
/** library version (string) */
#define LD10K1_LIB_VERSION_STR	"0.1.8p1"

