/*
 *  EMU10k1 loader
 *
 *  Copyright (c) 2003,2004 by Peter Zubaj
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
 
#ifndef __LO10K1_H
#define __LO10K1_H

#ifdef __cplusplus
extern "C" {
#endif

#include <lo10k1/version.h>

#include <lo10k1/ld10k1_error.h>
#include <lo10k1/ld10k1_fnc.h>
#include <lo10k1/comm.h>
#include <lo10k1/liblo10k1.h>
#include <lo10k1/liblo10k1ef.h>
#include <lo10k1/liblo10k1lf.h>

#ifdef __cplusplus
}
#endif

#endif /* __LO10K1_H */
